﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Programmable.Encodings {
  public sealed class MultiEncodingIndividual : Individual {
    public new MultiEncoding Encoding {
      get { return (MultiEncoding)base.Encoding; }
    }

    public MultiEncodingIndividual(IEncoding encoding, IScope scope)
      : base(encoding, scope) {
      if (encoding.GetType() != typeof(MultiEncoding))
        throw new ArgumentException("Encoding must be of type MultiEncoding.k");
    }

    [StorableConstructor]
    private MultiEncodingIndividual(bool deserializing) : base(deserializing) { }

    public override IDeepCloneable Clone(Cloner cloner) { return new MultiEncodingIndividual(this, cloner); }
    private MultiEncodingIndividual(Individual original, Cloner cloner) : base(original, cloner) { }

    public override IItem this[string name] {
      get {
        var encoding = Encoding.Encodings.FirstOrDefault(e => e.Name == name);
        if (encoding == null) throw new ArgumentException(string.Format("{0} is not part of the specified encoding.", name));
        return ExtractScopeValue(name, encoding.Name, Scope);
      }
      set {
        var encoding = Encoding.Encodings.FirstOrDefault(e => e.Name == name);
        if (encoding == null) throw new ArgumentException(string.Format("{0} is not part of the specified encoding.", name));
        SetScopeValue(name, encoding.Name, Scope, value);
      }
    }

    internal override Individual Copy(IScope scope) {
      var individual = Encoding.CreateIndividual(scope);
      foreach (var encoding in Encoding.Encodings)
        individual[encoding.Name] = this[encoding.Name];
      return individual;
    }
  }
}
